'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babylon = require('babylon');

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _s2sUtils = require('s2s-utils');

var _lodash = require('lodash.flatten');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import blog from 'babel-log'

/* ::
import type {Path, State} from 's2s-babel-flow-types'
*/

const builder = {
  test: (0, _s2sUtils.template)(`test(TITLE, () => {
    const result = FUNC()
    expect(result).toBe(null)
  })`)
};

const getFuncNames = (code /* :string */) => {
  const ast = (0, _babylon.parse)(code, {
    sourceType: 'module',
    plugins: ['flow', 'objectRestSpread']
  });

  const names /* :string[] */ = [];

  (0, _babelTraverse2.default)(ast, {
    FunctionDeclaration(path) {
      if (!path.findParent(p => p.isExportDeclaration())) {
        return;
      }
      const name = path.get('id').node.name;
      names.push(name);
    }
  });

  return names;
};

exports.default = () => {
  return {
    inherits: (0, _s2sUtils.inheritsOpts)(),
    name: 's2s-jest-unit-test-case',
    visitor: {
      Program(rootPath /* : Path */, { opts: { from }, file /* : State */ }) {
        if (!from) {
          throw new Error('required from option');
        }

        const targetPath = (0, _s2sUtils.getImportPath)(file.opts.filename, from);

        const code = _fs2.default.readFileSync(from, 'utf8');
        const names = getFuncNames(code);
        const nameMap /* : Set<string> */ = new Set(names);

        rootPath.traverse({
          ImportDeclaration(path) {
            if (path.get('source').node.value === targetPath) {
              path.remove();
            }
          },
          CallExpression(path) {
            const name = path.get('callee').node.name;
            nameMap.delete(name);
          }
        });

        const asts = Array.from(nameMap).map(name => [t.noop(), builder.test({
          TITLE: t.stringLiteral(name),
          FUNC: t.identifier(name)
        })]);

        rootPath.pushContainer('body', (0, _lodash2.default)(asts));

        function createImport(v) {
          const specifiers = names.map(name => {
            const i = t.identifier(name);
            return t.importSpecifier(i, i);
          });
          return t.importDeclaration(specifiers, t.stringLiteral(v));
        }

        rootPath.unshiftContainer('body', [createImport(targetPath)]);
      }
    }
  };
};